@@ -1255,7 +1255,7 @@ const char *cbmapiterval(const char *kbuf, int *sp){
   assert(kbuf);
   datum = (CBMAPDATUM *)(kbuf - sizeof(*datum));
   if(sp) *sp = datum->vsiz;
-  return (char *)datum + sizeof(*datum) + datum->ksiz + CB_ALIGNPAD(datum->ksiz);;
+  return (char *)datum + sizeof(*datum) + datum->ksiz + CB_ALIGNPAD(datum->ksiz);
 }
 
 
