@@ -538,6 +538,7 @@ public:
   //----------------------------------------------------------------
 private:
   CURIA *curia;                       ///< internal database handle
+  pthread_mutex_t mymutex;            ///< internal mutex
   //----------------------------------------------------------------
   // private member functions
   //----------------------------------------------------------------
@@ -546,6 +547,10 @@ private:
   Curia(const Curia& curia) throw(Curia_error);
   /** assignment operator: This should not be used. */
   Curia& operator =(const Curia& curia) throw(Curia_error);
+  /** lock the database. */
+  bool lock();
+  /** unlock the database. */
+  void unlock();
 };
 
 
