@@ -151,92 +151,98 @@ const char* Depot::version() throw(){
 
 
 void Depot::remove(const char* name) throw(Depot_error){
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(pthread_mutex_lock(&ourmutex) != 0) throw Depot_error();
   if(!dpremove(name)){
-    pthread_mutex_unlock(&mutex);
+    pthread_mutex_unlock(&ourmutex);
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  pthread_mutex_unlock(&ourmutex);
 }
 
 
 char* Depot::snaffle(const char* name, const char* kbuf, int ksiz, int* sp) throw(Depot_error){
   char* vbuf;
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(pthread_mutex_lock(&ourmutex) != 0) throw Depot_error();
   if(!(vbuf = dpsnaffle(name, kbuf, ksiz, sp))){
-    pthread_mutex_unlock(&mutex);
+    pthread_mutex_unlock(&ourmutex);
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  pthread_mutex_unlock(&ourmutex);
   return vbuf;
 }
 
 
 Depot::Depot(const char* name, int omode, int bnum) throw(Depot_error)
   : ADBM(){
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  pthread_mutex_init(&mymutex, NULL);
+  if(!lock()) throw Depot_error();
   if(!(depot = dpopen(name, omode, bnum))){
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  unlock();
 }
 
 
 Depot::~Depot() throw(){
-  if(!depot) return;
-  pthread_mutex_lock(&mutex);
-  dpclose(depot);
+  if(!depot){
+    pthread_mutex_destroy(&mymutex);
+    return;
+  }
+  if(lock()){
+    dpclose(depot);
+    unlock();
+  }
   depot = 0;
-  pthread_mutex_unlock(&mutex);
+  pthread_mutex_destroy(&mymutex);
 }
 
 
 void Depot::close() throw(Depot_error){
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   if(!dpclose(depot)){
     depot = 0;
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
   depot = 0;
-  pthread_mutex_unlock(&mutex);
+  unlock();
 }
 
 
 void Depot::put(const char* kbuf, int ksiz, const char* vbuf, int vsiz, int dmode)
   throw(Depot_error){
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   if(!dpput(depot, kbuf, ksiz, vbuf, vsiz, dmode)){
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  unlock();
 }
 
 
 void Depot::out(const char* kbuf, int ksiz) throw(Depot_error){
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   if(!dpout(depot, kbuf, ksiz)){
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  unlock();
 }
 
 
 char* Depot::get(const char* kbuf, int ksiz, int start, int max, int* sp) throw(Depot_error){
   char* vbuf;
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   if(!(vbuf = dpget(depot, kbuf, ksiz, start, max, sp))){
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  unlock();
   return vbuf;
 }
 
@@ -244,12 +250,12 @@ char* Depot::get(const char* kbuf, int ksiz, int start, int max, int* sp) throw(
 int Depot::getwb(const char *kbuf, int ksiz, int start, int max, char *vbuf) throw(Depot_error){
   int vsiz;
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   if((vsiz = dpgetwb(depot, kbuf, ksiz, start, max, vbuf)) == -1){
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  unlock();
   return vsiz;
 }
 
@@ -257,93 +263,93 @@ int Depot::getwb(const char *kbuf, int ksiz, int start, int max, char *vbuf) thr
 int Depot::vsiz(const char* kbuf, int ksiz) throw(Depot_error){
   int vsiz;
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   if((vsiz = dpvsiz(depot, kbuf, ksiz)) == -1){
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  unlock();
   return vsiz;
 }
 
 
 void Depot::iterinit() throw(Depot_error){
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   if(!dpiterinit(depot)){
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  unlock();
 }
 
 
 char* Depot::iternext(int* sp) throw(Depot_error){
   char* vbuf;
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   if(!(vbuf = dpiternext(depot, sp))){
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  unlock();
   return vbuf;
 }
 
 
 void Depot::setalign(int align) throw(Depot_error){
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   if(!dpsetalign(depot, align)){
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  unlock();
 }
 
 
 void Depot::setfbpsiz(int size) throw(Depot_error){
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   if(!dpsetfbpsiz(depot, size)){
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  unlock();
 }
 
 
 void Depot::sync() throw(Depot_error){
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   if(!dpsync(depot)){
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  unlock();
 }
 
 
 void Depot::optimize(int bnum) throw(Depot_error){
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   if(!dpoptimize(depot, bnum)){
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  unlock();
 }
 
 
 char* Depot::name() throw(Depot_error){
   char* buf;
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   if(!(buf = dpname(depot))){
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  unlock();
   return buf;
 }
 
@@ -351,12 +357,12 @@ char* Depot::name() throw(Depot_error){
 int Depot::fsiz() throw(Depot_error){
   int rv;
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   if((rv = dpfsiz(depot)) == -1){
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  unlock();
   return rv;
 }
 
@@ -364,12 +370,12 @@ int Depot::fsiz() throw(Depot_error){
 int Depot::bnum() throw(Depot_error){
   int rv;
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   if((rv = dpbnum(depot)) == -1){
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  unlock();
   return rv;
 }
 
@@ -377,12 +383,12 @@ int Depot::bnum() throw(Depot_error){
 int Depot::busenum() throw(Depot_error){
   int rv;
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   if((rv = dpbusenum(depot)) == -1){
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  unlock();
   return rv;
 }
 
@@ -390,12 +396,12 @@ int Depot::busenum() throw(Depot_error){
 int Depot::rnum() throw(Depot_error){
   int rv;
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   if((rv = dprnum(depot)) == -1){
-    pthread_mutex_unlock(&mutex);
+    unlock();
     throw Depot_error(dpecode);
   }
-  pthread_mutex_unlock(&mutex);
+  unlock();
   return rv;
 }
 
@@ -403,9 +409,9 @@ int Depot::rnum() throw(Depot_error){
 bool Depot::writable() throw(Depot_error){
   int rv;
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   rv = dpwritable(depot);
-  pthread_mutex_unlock(&mutex);
+  unlock();
   return rv ? true : false;
 }
 
@@ -413,9 +419,9 @@ bool Depot::writable() throw(Depot_error){
 bool Depot::fatalerror() throw(Depot_error){
   int rv;
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   rv = dpfatalerror(depot);
-  pthread_mutex_unlock(&mutex);
+  unlock();
   return rv ? true : false;
 }
 
@@ -423,9 +429,9 @@ bool Depot::fatalerror() throw(Depot_error){
 int Depot::inode() throw(Depot_error){
   int rv;
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   rv = dpinode(depot);
-  pthread_mutex_unlock(&mutex);
+  unlock();
   return rv;
 }
 
@@ -433,9 +439,9 @@ int Depot::inode() throw(Depot_error){
 time_t Depot::mtime() throw(Depot_error){
   time_t rv;
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   rv = dpmtime(depot);
-  pthread_mutex_unlock(&mutex);
+  unlock();
   return rv;
 }
 
@@ -443,9 +449,9 @@ time_t Depot::mtime() throw(Depot_error){
 int Depot::fdesc() throw(Depot_error){
   int rv;
   if(!depot) throw Depot_error();
-  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!lock()) throw Depot_error();
   rv = dpfdesc(depot);
-  pthread_mutex_unlock(&mutex);
+  unlock();
   return rv;
 }
 
@@ -497,5 +503,24 @@ Depot& Depot::operator =(const Depot& depot) throw(Depot_error){
 }
 
 
+bool Depot::lock(){
+  if(dpisreentrant){
+    if(pthread_mutex_lock(&mymutex) != 0) return false;
+    return true;
+  }
+  if(pthread_mutex_lock(&ourmutex) != 0) return false;
+  return true;
+}
+
+
+void Depot::unlock(){
+  if(dpisreentrant){
+    pthread_mutex_unlock(&mymutex);
+  } else {
+    pthread_mutex_unlock(&ourmutex);
+  }
+}
+
+
 
 /* END OF FILE */
