@@ -462,6 +462,7 @@ public:
   //----------------------------------------------------------------
 private:
   DEPOT* depot;                       ///< internal database handle
+  pthread_mutex_t mymutex;            ///< internal mutex
   //----------------------------------------------------------------
   // private member functions
   //----------------------------------------------------------------
@@ -470,6 +471,10 @@ private:
   Depot(const Depot& depot) throw(Depot_error);
   /** assignment operator: This should not be used. */
   Depot& operator =(const Depot& depot) throw(Depot_error);
+  /** lock the database. */
+  bool lock();
+  /** unlock the database. */
+  void unlock();
 };
 
 
