@@ -562,6 +562,7 @@ public:
   //----------------------------------------------------------------
 private:
   VILLA* villa;                       ///< internal database handle
+  pthread_mutex_t mymutex;            ///< internal mutex
   pthread_mutex_t tranmutex;          ///< mutex for the transaction
   //----------------------------------------------------------------
   // private member functions
@@ -571,6 +572,10 @@ private:
   Villa(const Villa& villa) throw(Villa_error);
   /** assignment operator: This should not be used. */
   Villa& operator =(const Villa& villa) throw(Villa_error);
+  /** lock the database. */
+  bool lock();
+  /** unlock the database. */
+  void unlock();
 };
 
 
